<?php

namespace Drupal\media_link\Plugin\Field\FieldWidget;

use Drupal\Core\Field\FieldDefinitionInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\media\MediaInterface;
use Drupal\media\MediaTypeInterface;
use Drupal\media_link\Plugin\media\Source\Link;
use Drupal\media_widget\Plugin\Field\FieldWidget\OEmbedMediaWidget;

/**
 * Extends the oEmbed media widget to support any remote
 */
class OEmbedLinkMediaWidget extends OEmbedMediaWidget {

  /**
   * {@inheritdoc}
   */
  protected static function isSupportedMediaType(MediaTypeInterface $media_type): bool {
    return parent::isSupportedMediaType($media_type) || static::isLinkMediaType($media_type);
  }

  /**
   * {@inheritdoc}
   */
  public static function validateInputElement(array $element, FormStateInterface $form_state, array $form): void {
    // If we have at least one Link media type, skip URL validation, as any
    // valid URL will be accepted.
    $field_definition = static::getFieldDefinition($element, $form_state);
    if (!static::hasLinkMediaType($field_definition)) {
      parent::validateInputElement($element, $form_state, $form);
    }
  }

  /**
   * {@inheritdoc}
   */
  protected static function getMediaTypeFromUrl(string $url, array $media_types): MediaTypeInterface {
    try {
      return parent::getMediaTypeFromUrl($url, $media_types);
    }
    catch (\InvalidArgumentException $e) {
      // Unsupported provider, we will consider this a plain link URL.
    }

    foreach ($media_types as $media_type) {
      if (static::isLinkMediaType($media_type)) {
        return $media_type;
      }
    }

    // This will only happen if no Link media type was enabled for the current
    // media reference field, so it is ok to rethrow the previous exception.
    throw $e;
  }

  /**
   * {@inheritdoc}
   */
  protected function createMediaFromValue(MediaTypeInterface $media_type, $value): MediaInterface {
    if (static::isLinkMediaType($media_type)) {
      try {
        $media = parent::createMediaFromValue($media_type, $value);
      }
      catch (\InvalidArgumentException $e) {
        // Invalid URL, fall back to an empty media item.
        $media_storage = $this->entityTypeManager
          ->getStorage('media');
        $media = $media_storage->create(['bundle' => $media_type->id()]);
        assert($media instanceof MediaInterface);
      }

      // By default populate the media name with the URL. This will be later
      // overridden by media settings.
      $media->setName($value);

      return $media;
    }
    else {
      return parent::createMediaFromValue($media_type, $value);
    }
  }

  /**
   * Checks whether the widget field references any Link media type.
   *
   * @param \Drupal\Core\Field\FieldDefinitionInterface $field_definition
   *   The widget field definition.
   *
   * @return bool
   *   TRUE if at least one link type is referenced, FALSE otherwise.
   */
  protected static function hasLinkMediaType(FieldDefinitionInterface $field_definition): bool {
    foreach (static::getMediaTypes($field_definition) as $media_type) {
      if (static::isLinkMediaType($media_type)) {
        return TRUE;
      }
    }
    return FALSE;
  }

  /**
   * Checks whether the specified media type is a Link one.
   *
   * @param \Drupal\media\MediaTypeInterface $media_type
   *   A media type entity.
   *
   * @return bool
   *   TRUE if this is a Link media type, FALSE otherwise.
   */
  protected static function isLinkMediaType(MediaTypeInterface $media_type): bool {
    return $media_type->getSource() instanceof Link;
  }

  /**
   * {@inheritdoc}
   */
  protected function buildInputElement(array $element, FormStateInterface $form_state, array $form): array {
    $element = parent::buildInputElement($element, $form_state, $form);
    $element['remote']['url']['#description'] = 'Enter URLs to link to an online document, video, or website.';

    return $element;
  }

}
