<?php

namespace Drupal\media_link\Plugin\media\Source;

use Drupal\Core\TypedData\Exception\MissingDataException;
use Drupal\link\LinkItemInterface;
use Drupal\media\MediaInterface;
use Drupal\media\MediaSourceBase;
use Drupal\media\MediaTypeInterface;

/**
 * Link media source.
 *
 * @MediaSource(
 *   id = "link",
 *   label = @Translation("Link"),
 *   description = @Translation("Use external links for reusable media."),
 *   allowed_field_types = {"link"},
 *   default_thumbnail_filename = "generic.png",
 * )
 */
class Link extends MediaSourceBase {

  /**
   * {@inheritdoc}
   */
  public function getMetadataAttributes() {
    return [
      'title' => $this->t('Resource title'),
      'url' => $this->t('Resource URL'),
      'created' => $this->t('Resource creation date'),
      'icon_url' => $this->t('Local URI of the icon'),
      'thumbnail_uri' => $this->t('Local URI of the thumbnail'),
      'thumbnail_width' => $this->t('Thumbnail width'),
      'thumbnail_height' => $this->t('Thumbnail height'),
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function getMetadata(MediaInterface $media, $attribute_name) {
    $metadata = [];

    $link = $this->getSourceFieldItem($media);
    if ($link) {
      $url = $link->getUrl()->toString();
      $metadata += [
        'title' => $link->title ?: $url,
        'url' => $url,
        'created' => '',
        'icon_uri' => '',
        'thumbnail_uri' => '',
        'thumbnail_width' => 0,
        'thumbnail_height' => 0,
      ];
    }

    return $metadata[$attribute_name] ?? NULL;
  }

  /**
   * Returns the media link item.
   *
   * @param \Drupal\media\MediaInterface $media
   *   A media entity.
   *
   * @return \Drupal\link\LinkItemInterface|null
   *   The link item or NULL if none is defined.
   */
  protected function getSourceFieldItem(MediaInterface $media): ?LinkItemInterface {
    $source_field = $this->configuration['source_field'];
    if (empty($source_field)) {
      throw new \RuntimeException('Source field for media source is not defined.');
    }

    $items = $media->get($source_field);
    if ($items->isEmpty()) {
      return NULL;
    }

    try {
      $field_item = $items->first();
      assert($field_item instanceof LinkItemInterface);
      return $field_item;
    }
    catch (MissingDataException $e) {
      return NULL;
    }
  }

  /**
   * {@inheritdoc}
   */
  public function createSourceField(MediaTypeInterface $type) {
    $field_definition = parent::createSourceField($type);

    $field_definition->set('settings', [
      'link_type' => LinkItemInterface::LINK_EXTERNAL,
      'title' => DRUPAL_DISABLED,
    ]);

    return $field_definition;
  }

}
